//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include <stdio.h>
#include "jetfight.h"


//                       **************************
//                      ***CONTROL SYSTEM CLASSES***
//                       **************************

ControlSystem::ControlSystem() { //load keyboard cfg, set up keyboard
}

ControlSystem::~ControlSystem() { //destructor
  timeleft = 0;
}

int ControlSystem::Init() { //load ControlSystem
  printf("  Installing Keyboard. . .\n");
  install_keyboard();
  clear_keybuf();

  c_left = get_config_int("controls", "turnleft", KEY_LEFT);
  c_right = get_config_int("controls", "turnright", KEY_RIGHT);
  c_up = get_config_int("controls", "accelerate", KEY_UP);
  c_down = get_config_int("controls", "deccelerate", KEY_DOWN);
  c_shoot = get_config_int("controls", "shoot", KEY_SPACE);
  c_nav = get_config_int("controls", "navsystem", KEY_N);
  c_pway = get_config_int("controls", "prevnavpoint", KEY_OPENBRACE);
  c_nway = get_config_int("controls", "nextnavpoint", KEY_CLOSEBRACE);
  c_auto = get_config_int("controls", "autopilot", KEY_A);

  printf("  Installing Joystick. . .  ");
  install_joystick(JOY_TYPE_AUTODETECT);
  poll_joystick();
  if (joy[0].num_buttons < 2) {
    joyinst = 0;
    printf("Joystick not found!\n");
    install_joystick(JOY_TYPE_NONE);
  } else {
    save_joystick_data("jetfight.cfg");
    joyinst = 1;
    printf("Joystick found: %i buttons, %i stick(s)\n", joy[0].num_buttons, joy[0].num_sticks);
  }

  printf("  Installing Mouse. . .  ");
  int temp = install_mouse(); //to get # of buttons
  if(temp == -1) {
    printf("\n**ERROR INSTALLING MOUSE, make sure mouse driver is installed!\n");
    return 1;
  } else {
    printf("%i buttons detected!\n", temp);
  }

  return 0;
}

void ControlSystem::Update() { //checks player input and makes approp changes
  if (joyinst) {            //keys OR joystick if it's installed
    poll_joystick();
    if (key[c_left] || joy[0].stick[0].axis[0].d1)
      Player.TurnLeft();
    else if (key[c_right] || joy[0].stick[0].axis[0].d2)
      Player.TurnRight();
    if (key[c_up] || joy[0].stick[0].axis[1].d1)
      Player.Accelerate();
    else if (key[c_down] || joy[0].stick[0].axis[1].d2)
      Player.Deccelerate();
    if (key[c_shoot] || joy[0].button[0].b)
      Player.Shoot(1);
    else
      Player.Shoot(0);
  } else {
    if (key[c_left])        //Left arrow turns
      Player.TurnLeft();
    else if (key[c_right])  //right arrow turns
      Player.TurnRight();
    if (key[c_up])          //up arrow accelerates
      Player.Accelerate();
    else if (key[c_down])   //down arrow deccelerates
      Player.Deccelerate();
    if (key[c_shoot])       //space bar shoots
      Player.Shoot(1);
    else                    //releasing the space bar stops shooting
      Player.Shoot(0);
  }

  //keys under time delay (switches, toggels)
  if (!timeleft) {
    if        (key[c_nav]) {  //"N" activates nav system
      Player.ActivateNavSystem();
      timeleft = 10;
    } else if (key[c_nway]) { //"]" next waypoint
      Player.NextWaypoint();
      timeleft = 10;
    } else if (key[c_pway]) { //"[" prev waypoint
      Player.PrevWaypoint();
      timeleft = 10;
    } else if (key[c_auto]) { //"A" autopilot
      Player.SetAutopilot();
      timeleft = 10;
    } else if (key[KEY_PRTSCR]) {//save .pcx
      Video.SaveScreen();
    }
  } else {
    timeleft--;
  }
}