//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include <stdio.h>
#include "jetfight.h"

//                       **************************
//                       ***VIDEO SYSTEM CLASSES***
//                       **************************

VideoSystem::~VideoSystem() {
  destroy_bitmap(BUFFER);
}

void VideoSystem::SaveScreen() {
  save_pcx("screen.pcx", BUFFER, (RGB *)data[jetfight].dat);
}

void dostars(int pos) {
  if (pos % 8 == 0) {
    printf("*");
    fflush(stdout);
  }
}

int VideoSystem::Init() { // loads graphics and sets up video system
  text_mode(-1);
  BUFFER = create_bitmap(scrx, scry);
  clear(BUFFER);

  interface = (FONT *)data[crystal].dat;
  console = font;

  printf("  Creating Trans Table: ");
  fflush(stdout);
  color_map = new COLOR_MAP;
  create_trans_table(color_map, (RGB *)data[jetfight].dat, 128, 128, 128, dostars);
  printf("\n");

  if (Parts.Init())
    return 1;

  return Sprites.Init();
}

int VideoSystem::EnterGraphics(int mode) { //enter graphics mode
  set_color_depth(mode);
  if (mode == 8) {
    if (set_gfx_mode(GFX_AUTODETECT, scrx, scry, 0, 0) == -1) {
      set_gfx_mode(GFX_TEXT, 80, 25, 0, 0);
      printf("**Error installing video driver!\n%s\n", allegro_error);
      readkey();
      return 1;
    }

    set_palette((RGB *)data[jetfight].dat);
    waterpal.r = 0;
    waterpal.g = 0;
    waterpal.b = 32;
    wateradd = 1;
  } else if (mode == 24) {
    if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) == -1) {
      set_gfx_mode(GFX_TEXT, 80, 25, 0, 0);
      printf("**Error installing video driver!\n%s\n", allegro_error);
      readkey();
      return 1;
    }
  } else if (mode == 32) {
    if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) == -1)
      return 1;
  }

  return 0;
}

void VideoSystem::SetView(int x, int y) { //sets middle of view to (x,y)
  viewx = x - scrx / 2;
  viewy = y - scry / 2;

  if (viewx < 0)
    viewx = 0;
  if (viewy < 0)
    viewy = 0;
  if (viewx > unitx - scrx)
    viewx = unitx - scrx;
  if (viewy > unity - scry)
    viewy = unity - scry;
}

void VideoSystem::DrawMessage(int x, int y, int color, char *s) {
  textout(BUFFER, font, s, x, y, color);
}

void VideoSystem::DrawInterface(PlayerStats stats, PlayerAmmo ammo, PlayerTrack track) {
  blit((BITMAP *)data[harticon].dat, BUFFER, 0, 0, 0, 0, 16, 16);
  rectfill(BUFFER, 32, 4, int(stats.life) + 32, 12, 116);

  blit((BITMAP *)data[bullicon].dat, BUFFER, 0, 0, 0, 32, 16, 16);
  textprintf(BUFFER, interface, 32, 32, 15, "%i", ammo.bullets);

  blit((BITMAP *)data[bombicon].dat, BUFFER, 0, 0, 0, 64, 16, 16);
  textprintf(BUFFER, interface, 32, 64, 15, "%i", ammo.bombs);

  blit((BITMAP *)data[missicon].dat, BUFFER, 0, 0, 0, 96, 16, 16);
  textprintf(BUFFER, interface, 32, 96, 15, "%i", ammo.missiles);
}

void VideoSystem::DrawNavSystem(PlayerTrack track, NavSystemData &navdata) {
//FUTURE OPTIMIZATION TIP -- put the nav system data in temporary vars --
//  the autopilot and the drawnav can both use them.
  if (navdata.type == 2) {
    AirEnemyTrack enemy = Enemies.GetAirTrack(navdata.targ);
    navdata.x = enemy.x;
    navdata.y = enemy.y;
  } else if (navdata.type) {
    GroundEnemyTrack enemy = Enemies.GetGroundTrack(navdata.targ);
    navdata.x = enemy.x;
    navdata.y = enemy.y;
  }

  draw_rle_sprite(BUFFER, (RLE_SPRITE *)data[navbackg].dat, 0, scry - 72);

  line(BUFFER, (int)track.x - viewx + 16, (int)track.y - viewy + 16, (int)navdata.x - viewx + 16, (int)navdata.y - viewy + 16, 8);

  textprintf(BUFFER, console, 4, scry - 64, 7, "Distance to target:%i",
    int(FindDistance(track.x, track.y, navdata.x, navdata.y) / 32));
  textprintf(BUFFER, console, 4, scry - 48, 7, "Waypoint set to   :%i, %i",
    int(navdata.x) / 32, int(navdata.y) / 32);
  textprintf(BUFFER, console, 4, scry - 32, 7, "Angle to waypoint :%i",
    int(FindAngle(track.x, track.y, navdata.x, navdata.y)));
  textprintf(BUFFER, console, 4, scry - 16, 7, "GPS ACTIVATED     :%i, %i",
    (int)track.x / 32, (int)track.y / 32);
}

void VideoSystem::DrawMap(MapStruct map[mapx][mapy]) {  //Draw the map to the scrn
  for (tempx = (viewx / 32); tempx <= ((viewx + scrx - 1) / 32); tempx++) {
    for (tempy = (viewy / 32); tempy <= ((viewy + scry - 1) / 32); tempy++) {
      blit((BITMAP *)data[map[tempx][tempy].image].dat, BUFFER, 0, 0, tempx * 32 - viewx, tempy * 32 - viewy, 32, 32);
    }
  }
}

void VideoSystem::DrawObject(int image, int x, int y, float angle) { //Draw Obj
  //ANGLE IN 360 DEGREE FORMAT!!!
  //Check to see if image is in the viewport
  if (x > viewx - 32 && x < viewx + scrx && y > viewy - 32 && y < viewy + scry) {
    tempangle = angle * 256.0 / 360.0;
    rotate_sprite(BUFFER, (BITMAP *)data[image].dat, x - viewx, y - viewy, ftofix(tempangle));
  } //end viewport if
}

void VideoSystem::DrawDoodad(RLE_SPRITE *image, int x, int y, char trans) { //draws a doodad
  if (x > viewx - 32 && x < viewx + scrx && y > viewy - 32 && y < viewy + scry) {
    if (trans)
      draw_trans_rle_sprite(BUFFER, image, x - viewx, y - viewy);
    else
      draw_rle_sprite(BUFFER, image, x - viewx, y - viewy);
  }
}

void VideoSystem::DrawBigPixel(int x, int y, int color) { //draws a pixel from unit map x,y
  int xloc = x - viewx;
  int yloc = y - viewy;

  if ((xloc > 0) && (xloc < scrx - 2) && (yloc > 0) && (yloc < scry - 2)) {
    _putpixel(BUFFER, xloc, yloc, color);
    _putpixel(BUFFER, xloc + 1, yloc, color);
    _putpixel(BUFFER, xloc + 1, yloc + 1, color);
    _putpixel(BUFFER, xloc, yloc + 1, color);
  }
}

void VideoSystem::DrawPixel(int x, int y, int color) { //draws a pixel from unit map x,y
  putpixel(BUFFER, x - viewx, y - viewy, color);
}

void VideoSystem::ShowText(int x, int y, int color, char *text) { //Shows texts at unit map coords
  textout(BUFFER, font, text, x - viewx, y - viewy, color);
}

void VideoSystem::Update() { //sends BUFFER to screen
//Rotate Water Palette -- water color is 111
  waterpal.b += wateradd;
  waterpal.g += wateradd;
  waterpal.r += wateradd;
  if (waterpal.b > 62)
    wateradd = -1;
  else if (waterpal.b < 33)
    wateradd = 1;
  set_color(111, &waterpal);

  Sprites.Update(); //Update doodads

  textprintf(BUFFER, font, scrx - 16, scry - 16, 15, "%i", fps * 5);
  blit(BUFFER, screen, 0, 0, 0, 0, scrx, scry);
}
