//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#ifndef __A_JETFIGHT_H__
#define __A_JETFIGHT_H__

#include <allegro.h>
#include <math.h>
#include "jetdata.h"

//defines for unused parts of allegro
//#define alleg_mouse_unused
//#define alleg_timer_unused
//#define alleg_keyboard_unused
#define alleg_joystick_unused
//#define alleg_gfx_driver_unused
//#define alleg_palette_unused
//#define alleg_graphics_unused
//#define alleg_vidmem_unused
#define alleg_flic_unused
//#define alleg_sound_unused
//#define alleg_file_unused
//#define alleg_datafile_unused
#define alleg_math_unused
#define alleg_gui_unused

//global vars
#define vol     255   //master volume
#define midivol 255   //MIDI volume
#define mapx    256   //tiles x, y
#define mapy    256
#define unitx   8192  //max unit x coord (mapx * 32)
#define unity   8192  //max unit y coord (mapx * 32)

#define scrx    640   //screen x
#define scry    480   //screen y
#define mscrx   1024  //map editor screen x/y
#define mscry   768

#define verno   "020a"

#define numbpart 250   //Max number of particles
#define pbull     25   //Max number of player bullets
#define ebull    100   //Max number of Enemy bullets

extern const float a2rad;

extern char joyinst;  //is joystick active?

extern DATAFILE *data;

extern volatile unsigned int fps;

float FindAngle(float x2, float y2, float x1, float y1);
  //angle of seg (Origin)-(destination)
inline float sq(float x) {return x*x;}
  //Finds x^2
inline float FindDistance(float x1, float y1, float x2, float y2) {
  return float(sqrt(sq(y2 - y1) + sq(x2 - x1)));}
  //distance from (x1,y1)-(x2,y2)
float FindDirection(float a1, float a2);
  //finds best path to angle a2 from angle a1

// ***MENUING SYSTEM CALLS***
extern int ret;

int Menus();    //Start Menuing System
int SetRet(void *dp3, int d2);
int d_check_exit(int msg, DIALOG *d, int c);

// ***MAP EDITOR CALL
int MapEdit();
void LayTiles(unsigned char x, unsigned char y, unsigned char image); //Lays tiles

unsigned char AtoD(unsigned char); //converts absolute maptiles to datafile indexes
unsigned char DtoAconv(unsigned char);
inline unsigned char DtoA(unsigned char d) { //lowers result to the TRUE answer
  return DtoAconv(d) - 1;
}

// ***Game Keys***
extern int c_left;
extern int c_right;
extern int c_up;
extern int c_down;
extern int c_shoot;
extern int c_nav;
extern int c_nway;
extern int c_pway;
extern int c_auto;

//                       **************************
//                       ***SOUND SYSTEM STRUCTS***
//                       **************************

//                     *****************************
//                     ***PARTICLE SYSTEM STRUCTS***
//                     *****************************

struct Particle {
  float x, y, changex, changey;
  char active;
  int color;
};

//                       ***************************
//                       ***PLAYER SYSTEM STRUCTS***
//                       ***************************

struct PlayerAmmo {
  int bullets;
  unsigned char bombs;
  unsigned char missiles;
  float bulletdamage, bombdamage, missiledamage;
};

struct PlayerTrack {
  float x, y, angle, speed; //cords & vector info
};

struct PlayerStats {
  unsigned char image; //image of the plane
  float turnrate, life, accel, minspeed, maxspeed; //ship stats
};

struct NavSystemData {
  unsigned char type; // type of loc: 0=land, 1=ground target, 2=air targ
  unsigned char targ; //enemy target number
  float x, y; // location of target
};

//                       **************************
//                       ***ENEMY SYSTEM STRUCTS***
//                       **************************

struct EnemyAmmo {
  int bullets;
  unsigned char bombs;
  unsigned char missiles;
  float bulletdamage, bombdamage, missiledamage;
};

struct AIScript {
  unsigned char action; //action?
   // 0=nothing
   // 1=goto position
   // 2=go away from position
   // 3=goto player
   // 4=go away from player
   // 5=attack player
   // 6=patrol
   // 7=defend area
   // 8=defend air target
  unsigned char target; //target? enemy number (patrol = other waypoint)
  float x, y; //location?
};

struct AirEnemyTrack {
  float x, y, angle, speed; //cords & vector info
};

struct AirEnemyStats {
  float turnrate, life, accel, minspeed, maxspeed; //ship stats
};

struct GroundEnemyTrack {
  float x, y, angle; //placement and angle info
};

struct GroundEnemyStats {
  float life;
};

struct AirEnemy {
  unsigned char type; //enemy type
  unsigned char image; //enemy image
  unsigned char onpoint; //what script(waypoint) is it on?
  unsigned char temp; //temp var used by AI system
  EnemyAmmo ammo;
  AirEnemyTrack track; //enemy position
  AirEnemyStats stats; //enemy stats
  AIScript AI[10]; //AI scripts
};

struct GroundEnemy {
  unsigned char type; //enemy type
  unsigned char image; //enemy image
  EnemyAmmo ammo;
  GroundEnemyTrack track; //enemy position
  GroundEnemyStats stats;
};

//                       **************************
//                       ***WORLD SYSTEM STRUCTS***
//                       **************************

struct MapStruct {
  unsigned char image; //image number
  unsigned char ground; //slot for ground enemies
};

//                       **************************
//                       ***WEAPONSYSTEM STRUCTS***
//                       **************************

struct Bullet {
  float x, y, changex, changey;
  float damage;
  char active;
  int color;
};

//                       **************************
//                      ***CONTROL SYSTEM STRUCTS***
//                       **************************

//                       *************************
//                       ***CLASS DECLARATIONS****
//                       *************************

class SoundSystem {
private:
//LOOPING DATA REQUIRED BY SOME SOUNDS
  unsigned char jetloop;
  unsigned char gunloop;
//Counter for time delays
  unsigned char multexp, multleft, multpan;
public:
  SoundSystem();             //Constructor Loads all sound data
  ~SoundSystem() {}          //Destructor
  int Init();                //loads sounds and sets up sfx system
  void Applause(int pan);    //plays Applause sound
  void AutopilotOn(int pan); //plays "Autopilot on" sound
  void Blip(int pan);        //plays blip/clicking sound
  void Bomb(int pan);        //plays falling bomb sound
  void Cannon(int pan);      //plays cannon sound
  void Explosion(int pan);    //plays a random explosion sound
  void MultExplosion(int pan, int length); //plays multiple explosions
  void Jet(int pan, int cmd, int freq); //plays the engine sounds for the jet
  void Laser(int pan);       //plays a laser sound effect
  void MachineGun(int pan, int cmd); //plays a machine gun sound
  void Missile(int pan);     //plays missile sound
  void MouseClick(int pan);  //plays mouse clicking sound
  void NavSystemOn(int pan); //plays nav system on sound
  void Neenee(int pan);      //plays funny little uhoh/neenee sound
  void SmallGunShot(int pan);//plays handgun sound
  void Welcome(int pan);     //plays voice, "Welcome."
  void Update();             //update sound system
// *** MUSIC ROUTINES ***
  void PlayMusic(int);       //plays a midi given a datafile index
  void MusicStop();          //stops any music that's playing
};

class Particles {
private:
  Particle parr[numbpart];
  short currparr;
public:
  ~Particles() {}
  Particles() {}
  int Init();
  void MakeNew(float x, float y, float angle, float speed, int color, char timeleft);
  void Update();
};

#define EXPLODE_SIZE    64
#define EXPLODE_FRAMES  48
#define EXPLODE_MAX     5

struct Explosion {
  short x, y;
  char active;
  char frame;
};

class Explosions {
private:
  DATAFILE *expl;
  char tot;
  char curr;
  Explosion expls[EXPLODE_MAX];
public:
  Explosions() {}
  ~Explosions() {unload_datafile(expl);}
  int Init();
  void MakeNew(short x, short y);
  void Update();
};

class SpritesSystem {
private:
  Explosions Exp;
public:
  SpritesSystem() {}
  ~SpritesSystem() {}
  int Init();
  void MakeNew(char type, short x, short y);
  void Update();
};

class VideoSystem {
private:
  //palette
  RGB waterpal; //cycle color for water
  char wateradd; //number to add to water

  //fonts
  FONT *interface;//Font for interface
  FONT *console;  //Font for console things

  //temp vars
  BITMAP *BUFFER; // buffer between code-->screen

  short int x, y; //temp vars for screen x,y
  short int tempx, tempy; //temporary vars for DrawMap();
  float tempangle;
  //view vars
  short int viewx, viewy; //upper left corner of viewport
public:
  VideoSystem() {} //constructor loads graphics
  ~VideoSystem(); //destructor frees graphics memory
  void SaveScreen(); //saves a .pcx screenshot
  int Init(); // loads graphics and sets up video system
  int EnterGraphics(int mode);  //enters graphics mode; loads pal
  void SetView(int x, int y); // sets middle of view to (x,y)
//Drawing Routines
  void DrawMessage(int x, int y, int color, char *s);
  void DrawInterface(PlayerStats stats, PlayerAmmo ammo, PlayerTrack track); //draws the interface on the bottom
  void DrawNavSystem(PlayerTrack track, NavSystemData &navdata);
  void DrawMap(MapStruct map[mapx][mapy]); //draws a terrain tile
  void DrawObject(int image, int x, int y, float angle);  //draws a sprite
  void DrawDoodad(RLE_SPRITE *image, int x, int y, char trans); //draws a doodad
//Primitive Draw routines on the Unit coord system
  void DrawBigPixel(int x, int y, int color); //draws a 320x200ish pixel
  void DrawPixel(int x, int y, int color); //draws a pixel from unit map x,y
  void ShowText(int x, int y, int color, char *text); //Displays text on the map

  void Update(); //sends buffer to screen
};

class PlayerSystem {  //keeps track of player location and stats
private:
  //plane data
  unsigned char playertype;    //player type
  PlayerTrack track; //plane tracking
  PlayerStats stats; //plane stats
  PlayerAmmo ammo; //ammo

  //Nav System Data
  unsigned char navactive; //is NavSystem on?
  unsigned char autopilot; //is autopilot on?
  unsigned char waypoints; //number of waypoints
  char onpoint; //what point are we on?

  NavSystemData navdata[10]; //nav waypoint list

  //misc data
  float tempdir, tempangle, tempdist; //temp vars

  inline void Calc(); //calculates new position
  void PlayerAI();
public:
  PlayerSystem();  //creates player
  ~PlayerSystem(); //destroys player
  int Init(int type, PlayerTrack track2, NavSystemData navdata2[10], PlayerAmmo ammo2, unsigned char waypoints2);
  void SetPos(short int x, short int y);
  void TurnLeft();
  void TurnRight();
  void Accelerate();
  void Deccelerate();
  void ActivateNavSystem();
  void NextWaypoint();
  void PrevWaypoint();
  void EnemyKilled(unsigned char index);//when an enemy dies, modify nav points
  void SetAutopilot();
  void Shoot(int cmd);
  int Damage(float x, float y, float damage); //Did we get hurt?
  void SetView();
  void Update();
// ***EDITOR PROCS
  void Load(PACKFILE *mapdata); //loads Player from a map
  void Save(PACKFILE *mapdata); //saves Player to a map
  void Draw(BITMAP *BUFFER, short int viewx, short int viewy);

  inline PlayerTrack GetTrack() {return track;}       //gets track
  inline PlayerAmmo GetAmmo() {return ammo;}          //gets ammo
  inline unsigned char PlayerType() {return playertype;}//gets playertype
  inline NavSystemData *GetNavData() {return navdata;}//gets navdata
  inline char GetNumbPoints() {return waypoints;}     //gets waypoints
  void SetNavData(NavSystemData nd[10], unsigned char numpoints);//saves navdata

  void GetData(PlayerTrack &track2, PlayerAmmo &ammo2, unsigned char &type2, NavSystemData navdata2[10], unsigned char &waypoints2);
  void SetData(PlayerTrack track2, PlayerAmmo ammo2, unsigned char type2, NavSystemData navdata2[10], unsigned char waypoints2);
};

class EnemySystem { //keeps track of all enemies
private:
  AirEnemy ae[50];
  GroundEnemy ge[50];
  unsigned char numbair;
  unsigned char numbground;
  unsigned char airkill[10];
  unsigned char groundkill[10];

  //temp vars
  short int c; //temp counter
  float tempdir, tempangle, tempdist; //temp vars
  PlayerTrack tempplayer;

  //AI scripts
  int TurnTo(int enemy, AirEnemy &ae, float x, float y);
    //turns enemy towards (x, y);
    //returns 1 if it's pointing towards it
  int MoveTo(int enemy, AirEnemy &ae, float x, float y);
    //moves enemy[enemy] towards (x,y)
    //returns 1 if target is within 6 squares
  int TurnAway(int enemy, AirEnemy &ae, float x, float y);
    //turns enemy towards (x, y);
    //returns 1 if it's pointing away from it
  int MoveAway(int enemy, AirEnemy &ae, float x, float y);
    //moves enemy[enemy] away from (x,y)
    //returns 1 if target is outside 10 squares
  inline int MoveToPlayer(int enemy, AirEnemy &ae);
    //moves enemy to the player
  inline int MoveAwayPlayer(int enemy, AirEnemy &ae);
    //moves enemy from the player
  inline void AttackPlayer(int enemy, AirEnemy &ae);
    //"attacks" player
  inline int Patrol(int enemy, AirEnemy &ae);
    //goes to the x,y, then processes command target

  void RunAI();      //Runs AI system on all targets
  void CalcAll();    //calcs all enemies
  void DisplayAll(); //displays all enemies
public:
  inline EnemySystem() {numbair = 0; numbground = 0;}
  inline ~EnemySystem() {}
  int Init();                   //clears out ae, ge arrays;
  void TurnLeft(int enemy);      //control an enemy
  void TurnRight(int enemy);
  void Accelerate(int enemy);
  void Deccelerate(int enemy);
  void AirShoot(int enemy);
  void GroundShoot(int enemy);
  void Update(PlayerTrack temp); //update System
// ***Weapons Procs***
  int DamageAirEnemies(float x, float y, float damage); //return 1 if a hit
  void DamageGroundEnemies(float x, float y, float damage);
// ***EDITOR PROCS***
  void Load(PACKFILE *mapdata);
  void Save(PACKFILE *mapdata);
  inline int GetNextAirEnemyNumb() {return numbair;}
  inline int GetNextGroundEnemyNumb() {return numbground;}
  void AddAirEnemy(int type, AirEnemyTrack track, EnemyAmmo ammo); //Adds an air enemy w/default AI
  void AddAirEnemy(int type, AirEnemyTrack track, EnemyAmmo ammo, AIScript AI[10]); //Adds an air enemy
  void AddGroundEnemy(int type, GroundEnemyTrack track, EnemyAmmo ammo); //adds a ground enemy
  void DeleteAirEnemy(char t);    //deletes air enemy [t]
  void DeleteGroundEnemy(char t); //deletes ground enemy [t]
  inline void ChangeAirEnemy(char c, AirEnemy rep) {ae[c] = rep;}       //replaces enemy [c]
  inline void ChangeGroundEnemy(char c, GroundEnemy rep) {ge[c] = rep;} //with rep
  void SetKills(unsigned char ak[10], unsigned char gk[10]);
  void GetKills(unsigned char ak[10], unsigned char gk[10]);
  void Draw(BITMAP *BUFFER, short int viewx, short int viewy);
// ***GET DATAS***
  char GetAirEnemy(short x, short y, AirEnemy &ret);      //returns enemy at x,y
  char GetGroundEnemy(short x, short y, GroundEnemy &ret);//and its index
  inline AirEnemyTrack GetAirTrack(int n) {return ae[c].track;} //returns a track
  inline GroundEnemyTrack GetGroundTrack(int n) {return ge[c].track;}
};

class WorldSystem { //keeps track of all objects
private:
  //map
  unsigned char miditoplay; //what midi does this map play?
  MapStruct map[mapx][mapy];

  //temp vars
public:
  WorldSystem() {}       //constructor loads world
  ~WorldSystem() {}      //destructor destroys world
  int Init();           //loads world
  void StartMusic();     //starts music miditoplay
  void Update();
// *** Editor Procs ***
  int Load(PACKFILE *mapdata);
  void Save(PACKFILE *mapdata);
  inline void ChangeMusic(int temp) {miditoplay = temp;} //changes miditoplay = int
  inline int GetMusic() {return miditoplay;}//gets current music number
  void Draw(BITMAP *BUFFER, short int viewx, short int viewy);
  void AddTile(unsigned char x, unsigned char y, unsigned char image);
  unsigned char GetTile(unsigned char x, unsigned char y);
};

class WeaponSystem {
private:
//Bullets
  Bullet pb[pbull]; //player's bullets
  Bullet eb[ebull]; //enemies' bullets

  short currpbull;
  short currebull;
//Missiles
  short currpmiss;
  short curremiss;
//Bombs
  short currbomb;
public:
  WeaponSystem() {}
  ~WeaponSystem() {}
  int Init();
  void MakeNewPlayer(float x, float y, float angle, float speed, int color, char timeleft, float damage);
  void MakeNewEnemy(float x, float y, float angle, float speed, int color, char timeleft, float damage);
  void Update();
};

class ControlSystem {
private:
  //key data
  //delay data
  unsigned char timeleft;
public:
   ControlSystem(); //load keyboard cfg, set up keyboard
   ~ControlSystem();  //destructor
   int Init(); //load ControlSystem
   void Update();
};

//Global Systems vars/class dec var

extern SoundSystem Sound;
extern VideoSystem Video;
extern Particles Parts;
extern SpritesSystem Sprites;
extern ControlSystem Control;
extern PlayerSystem Player;
extern EnemySystem Enemies;
extern WorldSystem World;
extern WeaponSystem Weapons;

#endif //_JETFIGHT_H_
